"""
Description:
   make connections with most of nosql DB servers.
"""

def _connect_to_mongodb():
    from pymongo import MongoClient
    CONNECTION_STRING = "mongodb://localhost:27017"
        # "mongodb+srv://<username>:<password>@<cluster-name>.mongodb.net/myFirstDatabase"
    try:
        MONGODBCON = MongoClient(CONNECTION_STRING)
    except:
        MONGODBCON = None



def __doc_sqlite3():
    import sqlite3

    LIGHTDB = sqlite3.connect('dasifo_default_storage.db')
    cur = LIGHTDB.cursor()

    # Create table
    cur.execute('''CREATE TABLE storages
                   (db_name text, host text, port text, username text, password text)''')

    # This is the qmark style:
    cur.execute("insert into lang values (?, ?)", ("C", 1972))

    # The qmark style used with executemany():
    lang_list = [
        ("Fortran", 1957),
        ("Python", 1991),
        ("Go", 2009),
    ]
    cur.executemany("insert into lang values (?, ?)", lang_list)

    # And this is the named style:
    cur.execute("select * from lang where first_appeared=:year", {"year": 1972})
    print(cur.fetchall())

    # Save (commit) the changes
    LIGHTDB.commit()

    # We can also close the connection if we are done with it.
    # Just be sure any changes have been committed or they will be lost.
    LIGHTDB.close()

import os
from .sql_db import _create_mysql
def create_connection(dbname=None, have=None):
    """  """
    DATABASES = ('mysql', 'postgresql', 'mongodb')
    DBKEYS = ('dbname', 'host', 'port', 'username', 'password')
    try:
        db_type = input(f"Enter one of these supported DBs ({DATABASES}): ")
        for k in DBKEYS:
            os.environ[db_type+'_'+k] = input(f"Enter {k}: ")

        if db_type == 'mysql':
            _create_mysql()


    except:
        pass


def find(con, table, id=None, one=False):
    pass
    sql = """  """
    cursor = con.cursor()
    cursor.execute(sql)
    return cursor.fetchall()


def update(con, table, id=None, have=None, by=None):
    pass


def insert(con, table, col_values=None):
    pass



"""
Description:
   make connections with most of sql DB servers:
   - MySQL
   - PostgreSQL
"""
import os
import subprocess

from mysql.connector import connect, Error
import psycopg2
from postgis.psycopg import register
# from postgis import LineString, Point, Polygon, MultiPoint, MultiPolygon, MultiLineString, Geometry


def _start_server(server_name=None):
    if server_name == 'mysql':
        bashCmd = ["ls", "."]
        process = subprocess.Popen(bashCmd, stdout=subprocess.PIPE)
        output, error = process.communicate()
    else:
        print("Not supported server")
def _create_mysql():
    try:
        # global
        MYSQLDBCON = connect(
            host=os.environ.get('mysql_host'),  # locaolhost
            port=os.environ.get('mysql_port'),  # 3306
            user=os.environ.get('mysql_username'),  #
            password=os.environ.get('mysql_password'),  # ''
            #database=os.environ.get('mysql_dbname')
        )

        # we have to check if the DBname if created or we recreate it
    except Error as e:
        print(e)
        MYSQLDBCON = None

def _create_postgresql():
    try:
        # global
        POSTGRESQLDBCON = psycopg2.connect(
                host=os.environ.get('postgresql_host'),  # locaolhost
                port=os.environ.get('postgresql_port'),  # 5432
                user=os.environ.get('postgresql_username'),  #
                password=os.environ.get('postgresql_password'),  # ''
                #database=os.environ.get('postgresql_dbname')
        )

        # we have to check if the DBname if created or we recreate it
        try:
            register(POSTGRESQLDBCON)
        except:
            # todo: create postgis and postgis_topology extensions for postgresql !
            print("there is no postgis extension in your postgresql server!")

        # cursor = POSTGRESQLDB.cursor()
    except:
        POSTGRESQLDBCON = None


import requests
from bs4 import BeautifulSoup

global URL
URL = "https://www.cora.fr/"


def set_url(url):
    """ define our default website url. """
    URL = url

def __request_html_page_and_parse(link):
    try:
        page = requests.get(link)
        return BeautifulSoup(page.content, "html.parser")
    except:
        print("HTML page not reachable! it could be not found or due to bad Network ... !")

def faire_mes_courses():
    """
        we want to list all products categories on this website.
        all produits are grouped by category for example: Animaux, Bébé, .....
        returns: array of categories that we will request for!
    """
    soup = __request_html_page_and_parse(URL)
    arr = []
    # find the navigation header of products
    lis = soup.nav.find_all("li", class_="c-list__item c-header-menu__nav-list-item")
    for li in lis:
        arr.append(li.find().attrs['href'])
    return arr


def sub_categories(of=None):
    """
    :param of: a string to be added to our url link then request html page
    :return: an array of subcategories that filters the products
    """
    if not isinstance(of, str):
        return []
    arr = []
    soup = __request_html_page_and_parse(URL+ of)
    lis = soup.nav.find_all("li", class_="c-list__item c-header-menu-layer__category")
    for li in lis:
        arr.append(li.find().attrs['href'])
    return arr

def get_products(link, only_priced=True):
    """
    :param link: url to be requested with HTTP (GET method)
    :param only_priced: True, False, None
        if True, we will only extract the products with a price.
        if False, extract the products which haven't a price.
        else, we return all the list of products.
    :return: a list of dicts, each dict represents a product described by : name, price, image
    """
    products = []
    soup = __request_html_page_and_parse(link)

    # we search for all products in the html page by looking to all li elements with class attribute
    by_class = "c-list__item c-product-list-container-products__item c-product-list-container-products__item--grid"
    li_products = soup.find_all("li",
                             class_=by_class)

    # we iterate all products found in our html page
    for pr in li_products:
        new_product = {}
        name = pr.find('a', class_="c-link-to c-product-list-item--grid__title c-link-to--hover-primary-light")
        new_product['name'] = name.get_text().replace("\n", ' ').strip()
        new_product['image'] = pr.find('img').attrs['src']
        try:  # is there any price in this product? if not, it ll throw an error
            price = pr.find('p', class_="c-price__amount")
            new_product['price'] = float(price.findChild().get_text().replace("\n", ' ').strip().replace(',', '.'))
        except:
            if only_priced == False:
                products.append(new_product)

        if only_priced != False and 'price' in new_product.keys():
            products.append(new_product)
    return products

def list_of_products(link="https://www.cora.fr/faire_mes_courses/animaux/chats/hygienesoin_et_accessoires_chat-c-176673", max_pages=None, only_priced=True):
    """
    we will ieterate all paginated pages that lists our products.
    :param link: url to be requested with HTTP (GET method)
    :param max_pages: limit the number of html pages to look at
    :param only_priced: True, False, None
        if True, we will only extract the products with a price.
        if False, extract the products which haven't a price.
        else, we return all the list of products.
    :return: a list of dicts, each dict represents a product described by : name, price, image
    """
    all_products = []
    i = 1
    if not isinstance(max_pages, int):
        # just trying to limit the number of html pages to look at
        max_pages = i
    while i!=0 and i <= max_pages: # ?? we can add a param to satisfy the limit number of paginations
        products = get_products(link+"?pageindex="+str(i), only_priced=only_priced)
        i += 1
        if any(products):
            all_products += products
        else:
            i == 0 # to exit the while loop because there are no products anymore
    return all_products

if __name__ == '__main__':
    print(list_of_products(max_pages=3))